#################################################################
# Gov 2001 Section 8 Code
# Overdispersion, Count Models, Duration Models
#################################################################
#setwd("~/Dropbox/Teaching/Course Folders/Gov 2001/Gov2001/Spring2016/Sections/Section 8")
# Generate Normal(mu, 1) data
  set.seed(02138)
  outcome <- rnorm(1000, 50, 1)
  
  setwd("~/Dropbox/Teaching/Course Folders/Gov 2001/Gov2001/Spring2016/Sections/Section 8")
  pdf("nhist1.pdf")
    hist(outcome, col = "dodgerblue2", border = "white", xlab = "Outcome Variable", main = "Normally Distributed Data")
  dev.off()
  
# Generate Normal(mu, sigma^2 > 1) data
  outcome2 <- rnorm(1000, 50, 7)
  pdf("nhist2.pdf")
    hist(outcome2, col = "dodgerblue2", border = "white", xlab = "Outcome Variable", main = "Normally Distributed Data", breaks = 20)
  dev.off()
  
# Generate Skewed Normal Data
  outcome3 <- sort(rnorm(1500, 50, 7))
  outcome3 <- outcome3[501:length(outcome3)]
  pdf("nhist3.pdf")
    hist(outcome3, col = "dodgerblue2", border = "white", xlab = "Outcome Variable", main = "Normally Distributed Data?", breaks = 10)
  dev.off()
  
# Generate Poisson Data
  seq <- rpois(10000, 3)
  library(MASS)
  pdf("pois.pdf")
    truehist(seq, xlab = "y", ylab = "Pr(Y=y)", prob = TRUE, h = .5,
             main = "Poisson Distribution", col = "dodgerblue2", border = "white", ylim = c(0,.5), 
             axes = FALSE)
    axis(2, at = seq(0,.5,length.out = 8), labels = seq(0,.175, .025))
    axis(1, at = seq(0,12,1), labels = c(0,NA,2,NA,4,NA,6,NA,8,NA,10,NA,12))
  dev.off()
  
  
# Poisson Log Likelihood Function
  pois.ll <- function(par, y, x){
    x <- as.matrix(cbind(1, x))
    xb <- x %*% par
    sum(y * xb - exp(xb))
  }
  
# Load Data on Terrorist Attacks
  load("attacks.RData")
  
# Find MLEs
  opt.pois <- optim(par = rep(0,2),
                    fn = pois.ll,
                    x = attacks$unemploy,
                    y = attacks$deaths,
                    hessian = T,
                    method = "BFGS",
                    control = list(fnscale = -1))
    
  opt.pois$par

# Create Diagnostic Plot for Overdispersion
  # Create Sequence of x - axis values to run through marginal effects functions
    unemp.seq <- seq(min(attacks$unemploy), max(attacks$unemploy), by = 0.001)
  
  #Save Diagnostic Plot    
    pdf("overdis.pdf")
    plot(attacks$unemploy, attacks$deaths, col = "cyan4", pch = 18, bty = "n", xlim = c(0.13, 0.35), axes = FALSE, xlab = "Unemployment Rate", ylab = "Terrorist Attacks", main = "Overdispersion in Poisson Data")
    axis(2, at = seq(0, 50, by = 5), labels = seq(0, 50, by = 5))
    axis(1, at = seq(0, 0.35, by = 0.05), labels = seq(0, 0.35, by = 0.05))
    lines(unemp.seq, exp(opt.pois$par[1] + opt.pois$par[2]*unemp.seq),
          lwd = 4, col = "grey59")
    lines(unemp.seq, qpois(.975, exp(opt.pois$par[1] + opt.pois$par[2]*unemp.seq)),
          lwd = 4, col = "grey59", lty = "dashed")
    lines(unemp.seq, qpois(.025, exp(opt.pois$par[1] + opt.pois$par[2]*unemp.seq)),
          lwd = 4, col = "grey59", lty = "dashed")
    dev.off()
    
    # Note that we're making a confidence interval for POINTS, not for lambda, so I'm using qpois to get
    # the 0.975 and .025 quantiles of the Poisson distribution with a mean at lambda_i for each
    # observation in the unemp.seq vector. Think of it as drawing a Poisson vertically at each x where the 
    # mean of the curved Poisson is at the solid line and the quantiles are at the dashes. 
    # The lines are discontinuous because qpois() necessarily returns integers.

# Another way to visually diagnose this is to look at the relative spread of your
# actual vs. predicted outcomes:
    pdf("checkspread.pdf")
    plot(attacks$unemploy, 
         attacks$deaths, 
         col = "cyan4", 
         pch = 18, 
         bty = "n", 
         xlim = c(0.13, 0.35), 
         axes = FALSE, 
         xlab = "Unemployment Rate", 
         ylab = "Terrorist Attacks", 
         main = "Overdispersion in Poisson Data")
    points(attacks$unemploy, rpois(length(attacks$unemploy),exp(cbind(1, attacks$unemploy)%*%opt.pois$par)),
           col = "lightpink1", pch = 18)
    axis(2, at = seq(0, 50, by = 5), labels = seq(0, 50, by = 5))
    axis(1, at = seq(0, 0.35, by = 0.05), labels = seq(0, 0.35, by = 0.05))
    legend("topright", c("Actual", "Predicted"), pch = 18, bty = "n", col = c("cyan4", "lightpink1"))
    dev.off()
    
    # Note that we have less dispersion in our predicted values than in our actual data.
    # That means our model is predicting less variation than we have, so we're overdispersed.
    
    
# You can also run your model as glm() object and check the ratio of your residual deviance (data variance
# not explained by your model) to your degrees of freedom (reduced by # params you've calculated)
    
    poismod <- glm(deaths ~ unemploy, family = "poisson", data = attacks)
    deviance(poismod)/poismod$df.residual # a value much > 1 here suggests overdispersion, and we have that problem
    
# The AER Package has a formal overdispersion test, but you'll need a glm() object: 
# This works in higher dimensional space where it's harder to plot.
    
  #install.packages("AER")
    require(AER)
    dispersiontest(poismod)
    
    # Null hypothesis here is that there is no dispersion. Reject the null
    
# Now let's check to see if we've solved the problem by using the negative binomial
# I'll use the glm.nb function in the MASS package to get negative binomial predictions
    mod.nb <- glm.nb(deaths ~ unemploy, data = attacks)
    summary(mod.nb)
    
# Now we can check our deviance:residual ratio
    deviance(mod.nb)/mod.nb$df.residual # That's basically 1, so we're not overdispersed!
    
# And plot our predicted values against our actual outcomes
    pdf("checkspreadnb.pdf")
    plot(attacks$unemploy, 
         attacks$deaths, 
         col = "cyan4", 
         pch = 18, 
         bty = "n", 
         xlim = c(0.13, 0.35), 
         axes = FALSE, 
         xlab = "Unemployment Rate", 
         ylab = "Terrorist Attacks", 
         main = "Overdispersion in Poisson Data")
    points(attacks$unemploy, rnbinom(n = length(attacks$unemploy), 
                                     size = mod.nb$theta, 
                                     mu = exp(cbind(1, attacks$unemploy)%*%mod.nb$coefficients)),
           col = "lightpink1", pch = 18)
    axis(2, at = seq(0, 50, by = 5), labels = seq(0, 50, by = 5))
    axis(1, at = seq(0, 0.35, by = 0.05), labels = seq(0, 0.35, by = 0.05))
    legend("topright", c("Actual", "Predicted"), pch = 18, bty = "n", col = c("cyan4", "lightpink1"))
    dev.off()
    